==> vocabulary_clone-1.0.0.tar.gz: vocabulary_clone/src/Controller/VocabularyCloneController.php <==
<?php

namespace Drupal\vocabulary_clone\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller for cloning a vocabulary and its terms.
 */
class VocabularyCloneController extends ControllerBase {

  /**
   * Clones a vocabulary and its terms.
   *
   * @param string $vocabulary_id
   *   The ID of the vocabulary to clone.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response.
   */
  public function cloneVocabulary($vocabulary_id) {
    // Load the vocabulary.
    $vocabulary = Vocabulary::load($vocabulary_id);
    if (!$vocabulary) {
      return new Response($this->t('Vocabulary not found: @vocabulary_id', ['@vocabulary_id' => $vocabulary_id]), 404);
    }

    if (strpos($vocabulary->label(), '(Clone)') !== FALSE) {
      \Drupal::messenger()->addWarning($this->t('This vocabulary has already been cloned and cannot be cloned again.'));

      // Redirect to the taxonomy listing page.
      return new RedirectResponse(Url::fromRoute('entity.taxonomy_vocabulary.collection')->toString());
    }

    // Clone the vocabulary.
    $new_vocabulary = $this->cloneVocabularyEntity($vocabulary);

    if (!$new_vocabulary) {
      return new Response($this->t('Failed to clone vocabulary: @vocabulary_id', ['@vocabulary_id' => $vocabulary_id]), 500);
    }

    // Clone the terms associated with the vocabulary.
    $this->cloneTerms($vocabulary, $new_vocabulary);

    // Redirect to the taxonomy listing page with a success message.
    \Drupal::messenger()->addStatus($this->t('Vocabulary %vocabulary cloned successfully, with terms.', ['%vocabulary' => $vocabulary->label()]));
    return new RedirectResponse(Url::fromRoute('entity.taxonomy_vocabulary.collection')->toString());
  }

  /**
   * Clones a vocabulary entity.
   *
   * @param \Drupal\taxonomy\Entity\Vocabulary $vocabulary
   *   The vocabulary to clone.
   *
   * @return \Drupal\taxonomy\Entity\Vocabulary|null
   *   The cloned vocabulary, or null if there was an error.
   */
  protected function cloneVocabularyEntity(Vocabulary $vocabulary) {
    try {
      \Drupal::logger('vocabulary_clone')->notice('Cloning vocabulary with ID: @vid', [
        '@vid' => $vocabulary->id(),
        '@vocabulary_name' => $vocabulary->label(),
      ]);

      $new_vocabulary = Vocabulary::create([
        'name' => $vocabulary->label() . ' (Clone)',
        'vid' => $vocabulary->id() . '_clone',
        'description' => $vocabulary->getDescription() ?: 'Cloned vocabulary',
      ]);

      // Save the new vocabulary.
      $new_vocabulary->save();

      \Drupal::logger('vocabulary_clone')->notice('Successfully cloned vocabulary. New vocabulary ID: @new_vid', [
        '@new_vid' => $new_vocabulary->id(),
        '@vocabulary_name' => $new_vocabulary->label(),
      ]);

      return $new_vocabulary;
    }
    catch (\Exception $e) {
      \Drupal::logger('vocabulary_clone')->error('Error cloning vocabulary: @error', [
        '@error' => $e->getMessage(),
        '@vocabulary_name' => $vocabulary->label(),
        '@vocabulary_id' => $vocabulary->id(),
      ]);

      // Return NULL.
      return NULL;
    }
  }

  /**
   * Clones the terms from the original vocabulary into the new vocabulary.
   *
   * @param \Drupal\taxonomy\Entity\Vocabulary $vocabulary
   *   The original vocabulary.
   * @param \Drupal\taxonomy\Entity\Vocabulary $new_vocabulary
   *   The new vocabulary to clone terms into.
   */
  protected function cloneTerms(Vocabulary $vocabulary, Vocabulary $new_vocabulary) {
    // Get all terms in the original vocabulary.
    $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadTree($vocabulary->id());

    foreach ($terms as $term) {
      // Load the term entity.
      $term_entity = Term::load($term->tid);
      if ($term_entity) {
        // Clone the term entity and associate it with the new vocabulary.
        $this->cloneTermEntity($term_entity, $new_vocabulary);
      }
    }
  }

  /**
   * Clones a term entity into the new vocabulary.
   *
   * @param \Drupal\taxonomy\Entity\Term $term
   *   The term entity to clone.
   * @param \Drupal\taxonomy\Entity\Vocabulary $new_vocabulary
   *   The new vocabulary to associate the cloned term with.
   */
  protected function cloneTermEntity(Term $term, Vocabulary $new_vocabulary) {
    try {
      $new_term = Term::create([
        'name' => $term->getName(),
        'vid' => $new_vocabulary->id(),
        'description' => $term->getDescription(),
      ]);

      $parent_terms = $term->get('parent')->getValue();
      if (!empty($parent_terms)) {
        $new_parent_ids = [];
        foreach ($parent_terms as $parent_term) {
          $new_parent_ids[] = $parent_term['target_id'];
        }

        $new_term->set('parent', $new_parent_ids);
      }
      $new_term->save();

      \Drupal::logger('vocabulary_clone')->notice('Cloned term: @term_name, New term ID: @new_tid', [
        '@term_name' => $new_term->label(),
        '@new_tid' => $new_term->id(),
      ]);
    }
    catch (\Exception $e) {
      \Drupal::logger('vocabulary_clone')->error('Error cloning term: @error', [
        '@error' => $e->getMessage(),
        '@term_name' => $term->label(),
        '@term_tid' => $term->id(),
      ]);
    }
  }

}

==> vocabulary_clone-1.0.3.tar.gz: vocabulary_clone/src/Controller/VocabularyCloneController.php <==
<?php

namespace Drupal\vocabulary_clone\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller for cloning a vocabulary and its terms.
 */
class VocabularyCloneController extends ControllerBase {

  /**
   * Clones a vocabulary and its terms.
   *
   * @param string $vocabulary_id
   *   The ID of the vocabulary to clone.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response.
   */
  public function cloneVocabulary($vocabulary_id, $new_machine_name) {
    // Load the original vocabulary.
    $vocabulary = Vocabulary::load($vocabulary_id);
    if (!$vocabulary) {
      return new Response($this->t('Vocabulary not found: @vocabulary_id', ['@vocabulary_id' => $vocabulary_id]), 404);
    }

    if (Vocabulary::load($new_machine_name)) {
      return new Response($this->t('The machine name @name is already taken.', ['@name' => $new_machine_name]), 400);
    }

    $new_vocabulary = $this->cloneVocabularyEntity($vocabulary, $new_machine_name);

    if (!$new_vocabulary) {
      return new Response($this->t('Failed to clone vocabulary: @vocabulary_id', ['@vocabulary_id' => $vocabulary_id]), 500);
    }

    $this->cloneTerms($vocabulary, $new_vocabulary);

    \Drupal::messenger()->addStatus($this->t('Vocabulary %vocabulary cloned successfully.', ['%vocabulary' => $new_machine_name]));
    return new RedirectResponse(Url::fromRoute('entity.taxonomy_vocabulary.collection')->toString());
  }

  /**
   *
   */
  protected function cloneVocabularyEntity(Vocabulary $vocabulary, $new_machine_name) {
    try {
      \Drupal::logger('vocabulary_clone')->notice('Cloning vocabulary with ID: @vid', [
        '@vid' => $vocabulary->id(),
        '@vocabulary_name' => $vocabulary->label(),
      ]);

      $new_vocabulary = Vocabulary::create([
        'name' => $new_machine_name,
        'vid' => $new_machine_name,
        'description' => $vocabulary->getDescription() ?: 'Cloned vocabulary',
      ]);

      // Save the new vocabulary.
      $new_vocabulary->save();

      \Drupal::logger('vocabulary_clone')->notice('Successfully cloned vocabulary. New vocabulary ID: @new_vid', [
        '@new_vid' => $new_vocabulary->id(),
        '@vocabulary_name' => $new_vocabulary->label(),
      ]);

      return $new_vocabulary;
    }
    catch (\Exception $e) {
      \Drupal::logger('vocabulary_clone')->error('Error cloning vocabulary: @error', [
        '@error' => $e->getMessage(),
        '@vocabulary_name' => $vocabulary->label(),
        '@vocabulary_id' => $vocabulary->id(),
      ]);

      return NULL;
    }
  }

  /**
   * Clones the terms from the original vocabulary into the new vocabulary.
   *
   * @param \Drupal\taxonomy\Entity\Vocabulary $vocabulary
   *   The original vocabulary.
   * @param \Drupal\taxonomy\Entity\Vocabulary $new_vocabulary
   *   The new vocabulary to clone terms into.
   */
  protected function cloneTerms(Vocabulary $vocabulary, Vocabulary $new_vocabulary) {
    // Get all terms in the original vocabulary.
    $terms = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->loadTree($vocabulary->id());

    foreach ($terms as $term) {
      // Load the term entity.
      $term_entity = Term::load($term->tid);
      if ($term_entity) {
        // Clone the term entity and associate it with the new vocabulary.
        $this->cloneTermEntity($term_entity, $new_vocabulary);
      }
    }
  }

  /**
   * Clones a term entity into the new vocabulary.
   *
   * @param \Drupal\taxonomy\Entity\Term $term
   *   The term entity to clone.
   * @param \Drupal\taxonomy\Entity\Vocabulary $new_vocabulary
   *   The new vocabulary to associate the cloned term with.
   */
  protected function cloneTermEntity(Term $term, Vocabulary $new_vocabulary) {
    try {
      $new_term = Term::create([
        'name' => $term->getName(),
        'vid' => $new_vocabulary->id(),
        'description' => $term->getDescription(),
      ]);

      $parent_terms = $term->get('parent')->getValue();
      if (!empty($parent_terms)) {
        $new_parent_ids = [];
        foreach ($parent_terms as $parent_term) {
          $new_parent_ids[] = $parent_term['target_id'];
        }

        $new_term->set('parent', $new_parent_ids);
      }
      $new_term->save();

      \Drupal::logger('vocabulary_clone')->notice('Cloned term: @term_name, New term ID: @new_tid', [
        '@term_name' => $new_term->label(),
        '@new_tid' => $new_term->id(),
      ]);
    }
    catch (\Exception $e) {
      \Drupal::logger('vocabulary_clone')->error('Error cloning term: @error', [
        '@error' => $e->getMessage(),
        '@term_name' => $term->label(),
        '@term_tid' => $term->id(),
      ]);
    }
  }

}
